#
# The MIT License
#
# @copyright Copyright (c) 2017 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#

#! /usr/bin/python
import struct
from threading import Thread
import sys
import os
import socket
import urllib
import time
import json

# VDMS Protobuf import (autogenerated)
from . import queryMessage_pb2

class vdms(object):

    def __init__(self):
        self.dataNotUsed = []
        self.conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.connected = False
        self.last_response = ''

    def __del__(self):
        self.conn.close()

    def connect(self, host='localhost', port=55555):
        self.conn.connect((host, port))
        self.connected = True

    def disconnect(self):
        self.conn.close()
        self.connected = False

    # Recieves a json struct as a string
    def query(self, query, img_array = []):

        # Check the query type
        if not isinstance(query, str): # assumes json
            query_str = json.dumps(query)
        else:
            query_str = query

        if not self.connected:
            return "NOT CONNECTED"

        quer = queryMessage_pb2.queryMessage()
        # quer has .json and .blob
        quer.json = query_str

        for im in img_array:
            quer.blobs.extend(im)

        # Serialize with protobuf and send
        # start = time.time()
        data = quer.SerializeToString();
        # start = time.time()
        sent_len = struct.pack('@I', len(data)) # send size first
        self.conn.send( sent_len )
        self.conn.send(data)
        # end = time.time()
        # print "ATcomm[ms]:" + str((end - start)*1000)

        # Recieve response
        recv_len = self.conn.recv(4)
        recv_len = struct.unpack('@I', recv_len)[0]
        response = b''
        while len(response) < recv_len:
            packet = self.conn.recv(recv_len - len(response))
            if not packet:
                return None
            response += packet

        querRes = queryMessage_pb2.queryMessage()
        querRes.ParseFromString(response)

        img_array = []
        for b in querRes.blobs:
            img_array.append(b)

        self.last_response = json.loads(querRes.json)

        return (self.last_response, img_array)

    def get_last_response(self):
        return self.last_response

    def get_last_response_str(self):
        return json.dumps(self.last_response, indent=4, sort_keys=False)
